"""
Usage:
    recover-utkpad-from-deltas.py <input_folder>

Description:
    This script reads all jpg images from <input_folder> and finds deltas (differences) with 
    images from the bonafide folder assumed to be in the parent directory.
    The resulted deltas are stored in the <output_folder> with relative paths preserved.

Arguments:
    input_folder      The path to the input folder with images.
                      [default: .]

Example:
    recover-utkpad-from-deltas.py ./
"""

from docopt import docopt
import os 
import cv2
import numpy as np
import glob
import gzip


def recover_images_from_deltas(deltas_root_folder):
    # assume a fixed name and path for bonadfide folder
    bonafide_folder = os.path.join(deltas_root_folder, 'bonafide')
    # assumer a fixed name for the delta folders, e.g., play-ipadpro-record-iphone12-deltas
    for deltas_folder in glob.glob(os.path.join(deltas_root_folder, '*-deltas')):
        for root, dirs, files in os.walk(deltas_folder):
            for filename in files:
                if filename.lower().endswith('.npz'):
                    # construct the paths requried to read deltas and bonafide images, 
                    # and reconstruct the attack images
                    delta_path = os.path.join(root, filename)
                    relative_path = os.path.relpath(delta_path, deltas_folder)
                    relative_path = os.path.splitext(relative_path)[0] + '.jpg'
                    bonafide_image_path = os.path.join(bonafide_folder, relative_path)
                    # folder with attack images will be the same but with no '-deltas' at the end
                    attack_folder = deltas_folder.split('-deltas')[0]
                    attack_image_path = os.path.join(attack_folder, relative_path)
                    print(f"recovering attack image from {delta_path} using {bonafide_image_path} ")
                    with gzip.open(delta_path, 'rb') as f:
                        im_delta = np.load(f)
                    im_bonafi = cv2.imread(bonafide_image_path)
                    height, width = im_delta.shape[:2]
                    im_bonafi_resized = cv2.resize(im_bonafi, (width, height), cv2.INTER_NEAREST)
                    # compute the attack image from delta and bonafide
                    im_attack = im_delta + im_bonafi_resized
                    print(f"Saving the attack image in {attack_image_path}")
                    os.makedirs(os.path.dirname(attack_image_path), exist_ok=True)
                    cv2.imwrite(attack_image_path, im_attack)


if __name__ == "__main__":
    arguments = docopt(__doc__)
    input_folder = arguments['<input_folder>']

    recover_images_from_deltas(input_folder)
    print(f"Finished recovering UTKPAD inside {input_folder}")
